#pragma once
#include "global_include.h"
#include "TextureLoader.h"

/*
 * README:
 * Unfortunately the HUD coordinate system is *not* defined
 * like a normal screen coordinate system.
 * Here we have (0,0) on the bottom left corner and top right
 * is (width, height). This is OpenGl dependent and cannot be changed.
 */

struct Label{
	GLint x;
	GLint y;

	const char *name;

	void *font;
	GLfloat fontColor[4];
};

struct Button {
	GLint x;
	GLint y;

	GLint dx;
	GLint dy;

	int ID;				// Button ID

	const char *name;
	glTexture *tex;		// path to picture

	void *font;
	GLfloat fontColor[4];
};

class HUD {

public:
	HUD(GLint width, GLint height);
	~HUD(void);

	void addLabel(GLint x, GLint y, const char *string, GLfloat fontColor[4] = null, void *font = null);
	void addButton(GLint x, GLint y, GLint dx, GLint dy, int ID, const char *string, const char *picture_path, GLfloat fontColor[4] = null, void *font = null);

	void drawHUD(void);

	void callback(void func(int)) ;
	int mouse(int button, int state, int x, int y);

private:
	void beginHUD(void);
	void endHUD(void);

	void drawLabels(void);
	void drawButtons(void);

private:
	GLint width;
	GLint height;

	GLfloat currentColor[4];
	GLboolean is_texuture_2d_enable;
//	GLboolean is_color_material_enable;

	vector<Label*>  labels;
	vector<Button*> buttons;

	void (*callback_func)(int);
};
