#pragma once
#include "global_include.h"

class Trackball
{
public:
	Trackball(GLuint button);
	~Trackball(void);

	//functions
	void animate(GLboolean animate);
	void idle_callback(void);
	void matrix(void);
	void motion(int x, int y);
	void mouse(int button, int state, int x, int y);
	void reshape(int w, int h);

private:
	//functions (internal)
	void _VEC_copy(const GLfloat *v1, GLfloat *v2);
	void _VEC_scale(GLfloat *v, GLfloat s);
	void _VEC_add(const GLfloat *v1, const GLfloat *v2, GLfloat *s);
	GLfloat _VEC_dot(const GLfloat *v1, const GLfloat *v2);
	void _VEC_cross(const GLfloat *v1, const GLfloat *v2, GLfloat *cross);
	void _VEC_project(GLfloat v[3], int x, int y);
	void _QUAT_norm(GLfloat quat[4]);
	void _QUAT_mult(GLfloat quat1[4], GLfloat quat2[4], GLfloat dest[4]);
	void _QUAT_buildQuat(GLfloat quat[4], GLfloat v[3]);
	void _QUAT_buildMatrix(GLfloat quat[4]);
	void _start(int x, int y, GLint time);
	void _stop(GLint time);

public:
	float radius;

private:
	//globals
	GLint     tb_timestamp;    //variable to hold time stamp
	GLuint    tb_width;        //stores viewer window width
	GLuint    tb_height;       //stores viewer window height
	GLuint    tb_button;       //default mouse button for rotation
	GLboolean tb_animate;      //flag indicating whether animation is enabled
	GLboolean tb_rotate;       //flag denoting ongoing rotation
	GLfloat   tb_matrix[4][4]; //current rotation matrix
	GLfloat   tb_lastquat[4];  //last quaternion
	GLfloat   tb_newquat[4];   //new quaternion
	GLfloat   tb_lastpos[3];   //last position on virtual trackball (3D vector)

	bool      execute_idle;
};
