// mass_spring.cpp : Defines the entry point for the console application.
//

#include "global_include.h"
// raytracer support (uncomment it to render with OpenGl)
#include "raytracer/opengl2ray.h"

#include "Time.h"
#include "glm.h"

#ifndef _OPENGL2RAY
#include "HUD.h"
#include "Trackball.h"
#endif




Time *time;
rayObjectLoader *object[11];

void setup_camera();

int model_list;

#ifndef _OPENGL2RAY
HUD *hud;
Trackball *tracker;
#endif

#define BTN_REV     0x01
#define BTN_PLAY    0x02
#define BTN_PAUSE   0x03

// prototypes
void callback_button(int id);


void reset(void) {

}

void init_opengl_scene(void) {
	float mat_specular[] = {1.0,1.0,1.0,1.0};
	float mat_shininess[] = {50.0};
	float light_position[] = {1.0,1.0,1.0,0.0};
	float white_light[] = {1.0,1.0,1.0,1.0};
	float lmodel_ambient[] = {0.4,0.5,0.5,1.0};

	// init time with hud
	time = new Time();
	time->pause();

#ifndef _OPENGL2RAY
	glClearColor(0.0,0.0,0.0,0.0);
	glShadeModel(GL_SMOOTH);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);

	// Init Trackball
	tracker = new Trackball(GLUT_MIDDLE_BUTTON);
	tracker->animate(GL_TRUE);
	tracker->radius = 4.0;

	// Init HUD
	hud = new HUD(800, 600);
	hud->addLabel(0,0, "HELLO WORLD!");
	hud->addButton(752, 144, 48, 22, BTN_REV, "", "graphics\\rev.gif");
	hud->addButton(752, 122, 48, 22, BTN_PLAY, "", "graphics\\play.gif");
	hud->addButton(752, 100, 48, 22, BTN_PAUSE, "", "graphics\\pause.gif");
	hud->callback(callback_button);

	GLfloat color[4] = {1.0f, 1.0f, 1.0f, 1.0f};
	hud->addLabel(20, 580, time->fps, color, GLUT_BITMAP_HELVETICA_12);

	ObjectLoader obj("models/whitestar.obj");
	obj.glmUnitize();
	obj.glmScale(3.0);
	obj.glmFacetNormals();
	obj.glmVertexNormals(45);

	model_list = obj.glmList(GLM_SMOOTH|GLM_TEXTURE);
#else
	rayPerspective(60, 1.0f);

	for(int i=0; i<11; ++i) {
		object[i] = new rayObjectLoader("models/whitestar.obj");
		object[i]->glmUnitize();
		object[i]->glmScale(3.0f);
		object[i]->glmFacetNormals();
		object[i]->glmVertexNormals(45);
	}

#endif
}

void redraw(void) {
	setup_camera();

#ifndef _OPENGL2RAY
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
		glColor3f(1.0,1.0,1.0);
		tracker->matrix();

		glEnable(GL_TEXTURE_2D);

		//call display list generated in 'func_create_list()'
		glPushMatrix();
		glTranslatef(0.0, 0.0, 0.0);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(3.5, 4.0, -8.0);
		glRotatef(25, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-7.0, -2.0, -19.0);
		glRotatef(-25, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(3.0, -2.0, -19.0);
		glRotatef(5, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(4.0, -2.0, -5.0);
		glRotatef(15, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-7.0, 4.0, -8.0);
		glRotatef(-7, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(0.0, 2.5, -15.0);
		glRotatef(-5, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-7.0, 2.0, -19.0);
		glRotatef(9, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-0.5, -3.5, -9.0);
		glRotatef(-38, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(4.0, 2.0, -14.0);
		glRotatef(42, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-5.0, -1.5, -2.0);
		glRotatef(-23, 0, 0, 1);
		glCallList(model_list);
		glPopMatrix();

		//glmDraw(model, material_mode);

		glutWireSphere(3.0, 60, 60);

		if (!time->is_paused) {
			// tetrahedron
//			tetrahedron->calculateForces();
//			tetrahedron->doStep(0.01f);
		}

		// draw mass-spring system
//		tetrahedron->draw();

		hud->drawHUD();
	glPopMatrix();	
	
	glutSwapBuffers();
#else
		float rot[11] = {
			0,
			25,
			-25,
			5,
			15,
			-7,
			-5,
			9,
			-38,
			42,
			-23};

		float trans[11][3] = {
			0.0, 0.0, 0.0,
			3.5, 4.0, -8.0,
			-7.0, -2.0, -19.0,
			3.0, -2.0, -19.0,
			4.0, -2.0, -5.0,
			-7.0, 4.0, -8.0,
			0.0, 2.5, -15.0,
			-7.0, 2.0, -19.0,
			-0.5, -3.5, -9.0,
			4.0, 2.0, -14.0,
			-5.0, -1.5, -2.0};


		for(int i=0; i<11; ++i) {
			primitives.push_back(object[i]->List(GLM_SMOOTH|GLM_TEXTURE));
			object[i]->rotate(rot[i], 0.0, 0.0, 1.0);
			object[i]->translate(trans[i]);
		}
#endif
}

// CALLBACKS
#ifndef _OPENGL2RAY
void callback_key (unsigned char key, int x, int y){
	if (key == 'r'){
		callback_button(BTN_REV);
	}
}

void callback_idle(void) {
	time->process();	

	if (!time->is_paused) {
		redraw();
	}

	//tracker->idle_callback();
}

void callback_motion(int x, int y) {
	//give control to trackball module
	tracker->motion(x, y);
}

void callback_mouse(int button, int state, int x, int y) {
	if (!hud->mouse(button, state, x, y)) {
		//give control to trackball module
		tracker->mouse(button, state, x, y);
	}
}

void callback_reshape(int w, int h) {
	//give reshape information to trackball module
	tracker->reshape(w, h);

	// rest
	if(h == 0)
		h = 1;

	float ratio = 1.0* w / h;

	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Set the viewport to be the entire window
	glViewport(0, 0, w, h);

	// Set the correct perspective.
	gluPerspective(60, ratio, 1.0, 128.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -8.0);
	//gluLookAt(0.0,0.0,7.0,
	//	0.0,0.0,-1.0,
	//	0.0f,1.0f,0.0f);
}

void callback_button(int id) {
	switch(id) {
		case BTN_PAUSE:
			time->pause();
			break;
		case BTN_PLAY:
			time->play();
			break;
		case BTN_REV:
			time->pause();
			break;
	}
}
#endif

void setup_camera() {
#ifndef _OPENGL2RAY
	glLoadIdentity();

	// Set the correct perspective.
//	gluPerspective(60, ratio, 1.0, 128.0);
	gluLookAt(1.0f, -0.3f, 4.0f,		// eye
	          0.0f, 0.0f, 0.0f,			// center
	          0.0f, 1.0f, 0.0f);		// up
#else
	rayLookAt(1.0f, -0.3f, 4.0f,		// eye
	          0.0f, 0.0f, 0.0f,			// center
	          0.0f, 1.0f, 0.0f);		// up
#endif
}

void main(int argc, char **argv) {
#ifndef _OPENGL2RAY
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800,600);

	glutCreateWindow("Test");

	// Callbacks
	glutDisplayFunc(redraw);
//	glutIdleFunc(redraw);
	glutKeyboardFunc(callback_key);
	glutIdleFunc(callback_idle);
	glutMotionFunc(callback_motion);
	glutMouseFunc(callback_mouse);
	glutReshapeFunc(callback_reshape);
	
	init_opengl_scene();
	glutMainLoop();
#else
	opengl2ray();
#endif
}